/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2013  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.88f                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : Start_Extended_OS_TASK.c
Purpose : Skeleton program for OS to demonstrate extended tasks
          This sample application is described in the generic manual
          in chapter:
          "Extending the task context by using own task structures"

--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"
#include <stdio.h>

/****** Define an own task structure with extended task context *****/

typedef struct {
  OS_TASK Task;     // OS_TASK has to be the first element
  OS_TIME Timeout;  // Any other data may follow
  char*   pString;
} MY_APP_TASK;

/****** Variables ***************************************************/

OS_STACKPTR int StackHP[128], StackLP[128];          /* Task stacks */
MY_APP_TASK     TCBHP, TCBLP;                /* Task-control-blocks */

/*********************************************************************
*
*       Task function
*/
static void MyTask(void) {
  char*        pString;
  OS_TIME      Delay;
  MY_APP_TASK* pThis;
  pThis = (MY_APP_TASK*) OS_GetTaskID();
  while (1) {
    Delay   = pThis->Timeout;
    pString = pThis->pString;
    printf(pString);
    OS_Delay (Delay);
  }
}

/*********************************************************************
*
*       main
*
*********************************************************************/

int main(void) {
  OS_IncDI();                      /* Initially disable interrupts  */
  OS_InitKern();                   /* Initialize OS                 */
  OS_InitHW();                     /* Initialize Hardware for OS    */
  /*
  *  Create the extended tasks just as normal tasks.
  *  Note that the first paramater has to be of type OS_TASK
  */
  OS_CREATETASK(&TCBHP.Task, "HP Task", MyTask, 100, StackHP);
  OS_CREATETASK(&TCBLP.Task, "LP Task", MyTask,  50, StackLP);
  /*
  *  Give task contexts individual data
  */
  TCBHP.Timeout = 200;
  TCBHP.pString = "HP task running\n";
  TCBLP.Timeout = 500;
  TCBLP.pString = "LP task running\n";
  OS_Start();                      /* Start multitasking            */
  return 0;
}
