/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2013  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.88f                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c (for Microchip PICDEM HPC Eplorer2 and C18 compiler)
Purpose : BSP for PICDEM Eplorer 2 evalboard
--------  END-OF-HEADER  ---------------------------------------------
*/

#define BSP_C
#include "BSP.h"
#include "RTOS.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Addresses of sfrs and peripheral identifiers ****************/
#define PORTD    (*(volatile unsigned char *) (0xF83))   // Port D data register 
#define LATD     (*(volatile unsigned char *) (0xF8C))   // Port D latch register 
#define TRISD    (*(volatile unsigned char *) (0xF95))   // Port D data direction register 

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       LED_Init()
*/
void BSP_Init(void) {
  TRISD = 0x00; // Set all Led port pins to input
  LATD  = 0x00; // Turn off all Leds
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with low level on port lines
*/
void BSP_SetLED(int Index) {
  LATD |= (1 << Index);
}

void BSP_ClrLED(int Index) {
  LATD &= ~(1 << Index);
}

void BSP_ToggleLED(int Index) {
  if (PORTD & (1 << Index)) {
    LATD &= ~(1 << Index);
  } else {
    LATD |= (1 << Index);
  }  
}

/****** EOF *********************************************************/


