/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2013  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.88f                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSINIT_PIC18F87J11.c
          for Microchip PIC18 (PIC18F87J11)

Purpose : Initializes and handles the hardware for the OS as far as 
          required by the OS.
          Feel free to modify this file acc. to your target system.

--------  END-OF-HEADER  ---------------------------------------------
*/


#include "RTOS.h"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/
#define ALLOW_NESTED_INTERRUPTS  0  // Caution: Nesting interrupts will cause higher stack load on system stack CSTACK
#ifndef   OS_USE_PLL
  #define OS_USE_PLL             0  // No PLL by default because there seems to be problems with RAM access when PLL is enabled
#endif

/*********************************************************************
*
*       Device Configuration
*
**********************************************************************
*/
#pragma config WDTEN = OFF
#pragma config XINST = ON
#if (OS_USE_PLL == 1)
  #pragma config FOSC = HSPLL
#else
  #pragma config FOSC = HS
#endif

/*********************************************************************
*
*       Clock frequency settings
*/
#if (OS_USE_PLL == 1)
  #define OS_FSYS (40000000L)
#else
  #define OS_FSYS (10000000L)
#endif

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ (1000)
#endif

#ifndef   OS_PCLK_TIMER
  #define OS_PCLK_TIMER  (OS_FSYS/4)
#endif

/*********************************************************************
*
*       Configuration of communication to OSView
*/
#ifndef   OS_VIEW_ENABLE            // Global enable of communication
  #define OS_VIEW_ENABLE    (1)     // Default: on
#endif

/*********************************************************************
*
*       UART settings for OSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to OSView, please define it to be -1
*       Currently UART0 and UART1 are supported and the standard
*       setup enables UART 0 per default
*/
#ifndef   OS_UART
  #define OS_UART (0)
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE (38400)
#endif

#ifndef   OS_PCLK_UART
  #define OS_PCLK_UART  (OS_FSYS/4)
#endif

/****** End of configuration settings *******************************/

/*********************************************************************
*
*       UART settings for OSView
*
*       Automatically generated from configuration settings
*/
#define OS_UART_USED   (OS_VIEW_ENABLE && (OS_UART == 0))

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
#define RCON        (*(volatile OS_U8*) (0xFD0))  // Reset Control Register
#define INTCON3     (*(volatile OS_U8*) (0xFF0))  // Interrupt Control Register 3
#define INTCON2     (*(volatile OS_U8*) (0xFF1))  // Interrupt Control Register 2
#define INTCON      (*(volatile OS_U8*) (0xFF2))  // Interrupt Control Register
#define PIE1        (*(volatile OS_U8*) (0xF9D))  // Peripherial Interrupt Enable Register 1
#define PIE2        (*(volatile OS_U8*) (0xFA0))  // Peripherial Interrupt Enable Register 2
#define PIR1        (*(volatile OS_U8*) (0xF9E))  // Peripherial Interrupt Request (Flag) Register 1
#define PIR2        (*(volatile OS_U8*) (0xFA1))  // Peripherial Interrupt Request (Flag) Register 2
#define PIR3        (*(volatile OS_U8*) (0xFA4))  // Peripherial Interrupt Request (Flag) Register 3
#define IPR1        (*(volatile OS_U8*) (0xF9F))  // Peripherial Priority Register 1
#define T0CON       (*(volatile OS_U8*) (0xFD5))  // Timer 0 Control Register
#define TMR0L       (*(volatile OS_U8*) (0xFD6))  // Timer 0 Value Low Byte
#define TMR0H       (*(volatile OS_U8*) (0xFD7))  // Timer 0 Value High Byte
#define T1CON       (*(volatile OS_U8*) (0xFCD))  // Timer 1 Control Register
#define TMR1L       (*(volatile OS_U8*) (0xFCE))  // Timer 1 Value Low Byte
#define TMR1H       (*(volatile OS_U8*) (0xFCF))  // Timer 1 Value High Byte
#define OSCTUNE     (*(volatile OS_U8*) (0xF9B))  // Oscillator tuning register
#define CCP1CON     (*(volatile OS_U8*) (0xFBB))  // Compare Condition register
#define CCPR1L      (*(volatile OS_U8*) (0xFBC))  // Compare value low register
#define CCPR1H      (*(volatile OS_U8*) (0xFBD))  // Compare value low register
#define RCSTA1      (*(volatile OS_U8*) (0xFAC))  // Receive Status and Control register
#define TXSTA1      (*(volatile OS_U8*) (0xFAD))  // Receive Status and Control register
#define TRISC       (*(volatile OS_U8*) (0xF94))  // Port C direction register
#define BAUDCON1    (*(volatile OS_U8*) (0xF7E))  // Baud settings register
#define SPBRGH1     (*(volatile OS_U8*) (0xF7F))  // Baud rate value high
#define SPBRG1      (*(volatile OS_U8*) (0xFB0))  // Baud rate value low
#define TXREG1      (*(volatile OS_U8*) (0xFAE))  // Transmit shift register
#define RCREG1      (*(volatile OS_U8*) (0xFAF))  // Receive register

#define INTERRUPT_PRIORITY_ENABLE (1 << 7)        // Interrupt Priority Enable
#define CCP1IF                    (1 << 2)        // Compare match flag
#define SPEN                      (1 << 7)        // Serial port enable bit
#define TXEN                      (1 << 5)        // Transmit enable bit
#define BRGH                      (1 << 2)        // Select high speed for baudrate
#define CREN                      (1 << 4)        // Continous receive enable bit
#define BRG16                     (1 << 3)        // 16 bit baud rate enable 
#define TX1IE                     (1 << 4)        // Tx1 Interrupt enable
#define RC1IE                     (1 << 5)        // Rx1 Interrupt enable
#define CCP1IE                    (1 << 2)        // Compare match enable flag
#define TX1IF                     (1 << 4)        // EUSART1 Transmit interrupt flag
#define RC1IF                     (1 << 5)        // EUSART1 Transmit interrupt flag
#define FERR                      (1 << 2)        // Framing error
#define OERR                      (1 << 1)        // Overrun error
#define TXIP                      (1 << 4)        // Tx1 interrrupt priority
#define RC1IP                     (1 << 5)        // Rx1 interrrupt priority
#define TMR0IF                    (1 << 2)

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/
#define OS_TIMER_RELOAD (OS_PCLK_TIMER/OS_TICK_FREQ)
#if (OS_TIMER_RELOAD >= 0x10000)
  #error "TMR1 timer can not be used, please check configuration"
#endif

void OS_InitHW(void) {
  OS_IncDI();
  // 
  // Set interrupt priority mode
  //
  RCON |= INTERRUPT_PRIORITY_ENABLE;   
  //
  // Enable PLL
  //
#if (OS_USE_PLL == 1)
  OSCTUNE |= (1 << 6);
#endif
  //
  // Initialize embOS timer
  //
  TMR1H   = 0;
  TMR1L   = 0;
  CCPR1L  = OS_TIMER_RELOAD & 0xFF;
  CCPR1H  = OS_TIMER_RELOAD >> 8;
  CCP1CON = 0x0B;         // Compare Match mode
  T1CON   =  (1 << 7);    // 16 bit timer, Prescaler 1:1
  PIE1   |=  CCP1IE;      // Timer1 compare interrupt enable  
  IPR1   &=~ (1 << 2);    // Low Interrupt  
  T1CON  |=  (1 << 0);    // Start timer
  OS_COM_Init();
  OS_DecRI();
}

/*********************************************************************
*
*       Idle loop  (OS_Idle)
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {           // Idle loop: No task is ready to exec 
  while (1) {
  }
}
  
/*********************************************************************
*
*        _OS_SystemIrqhandler
*/
static void _OS_SystemIrqhandler(void) {
  PIR1 &= ~CCP1IF;
  OS_TICK_Handle();  
}
  
/*********************************************************************
*
*       Get time [cycles]
*
*       This routine is required for task-info via OSView or high
*       resolution time maesurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32  Time;
  OS_U16  Cnt;
  OS_U16  CntHigh;  
  OS_U8   CntLow;
    
  Time    = OS_GetTime32();
  CntLow  = TMR1L;  
  CntHigh = TMR1H;  
  Cnt     = (CntHigh << 8) | CntLow;
  //  
  // Check if timer interrupt pending ...
  // 
  if (PIR1 & CCP1IF) {  // Adjust result
    CntLow  = TMR1L;
    CntHigh = TMR1H;
    Cnt     = (CntHigh << 8) | CntLow;
    Time++;
  }
  return (OS_TIMER_RELOAD * Time) + Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       Communication for OSView via UART (optional)
*
**********************************************************************
*/

#if OS_UART_USED

#define BAUDDIVIDE (OS_PCLK_UART / OS_BAUDRATE)

/*********************************************************************
*
*       OS_COM_ISR_Rx() 
*
*       OS USART Rx interrupt handler 
*/
void OS_COM_ISR_Rx(void) {
  OS_U8 c;
  PIR1 &= ~RC1IF;
  c = RCREG1;
  if (((RCSTA1 & FERR) == 0) && ((RCSTA1 & OERR) == 0)) {  // No Error occured?
    OS_OnRx(c);
  } else {
    RCSTA1 &= ~CREN;   // Clear erros by disabling and enabling receiver
    RCSTA1 |= CREN;
  }
}

/*********************************************************************
*
*       OS_COM_ISR_Tx() 
*
*       OS USART Tx interrupt handler 
*/
void OS_COM_ISR_Tx(void) {
  if (OS_OnTx()) {      // No more characters to send?  
     PIE1 &= ~TX1IE;    // Disable Transmission Interrupt Enable
  }
}

/*********************************************************************
*
*       OS_COM_Send1()
*       Never call this function directly from your application
*/
void OS_COM_Send1(OS_U8 c) {
  TXREG1 = c;  
  PIE1  |= TX1IE;   // Enable Transmit Interrupt 
}

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART for OSView
*/
void OS_COM_Init(void) {       // Initialize UART, enable UART interrupts 
  TRISC   |= (1 << 7);         // Set Tx Portin as Output
  TRISC   &= ~(1 << 6);        // Set Rx Portin as Input
  BAUDCON1 = BRG16;            // Set baudrate control register 
  SPBRGH1  = BAUDDIVIDE >> 8;  // Set baudrate
  SPBRG1   = BAUDDIVIDE & 0xFF;
  PIE1    |= RC1IE;            // Enable Transmit Interrupt 
  IPR1    &= ~(TXIP | RC1IP);  // Set Low interrupt priority for rx and tx interrupt
  TXSTA1   = TXEN | BRGH;      // Set high speed baudrate and enable tx 
  RCSTA1   = SPEN | CREN;      // Configure rx and tx portpins and enable rx
}

#else 

void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  (void) c;                    // Avoid compiler warning 
  OS_COM_ClearTxActive();      // Let the OS know that Tx is not busy 
}

#endif /*  OS_UART_USED  */

/****** Final check of configuration ********************************/

#ifndef OS_UART_USED
  #error "OS_UART_USED has to be defined"
#endif

/*********************************************************************
*
*        Interrupt functions
*
**********************************************************************
*/
#pragma interruptlow OS_irq_handler
void OS_irq_handler(void) {
#if (ALLOW_NESTED_INTERRUPTS == 1)
  OS_EnterNestableInterrupt();
#else
  OS_EnterInterrupt();
#endif  
  //
  // Check if interrupt flag is set and interrupt is enabled
  // 
  if ((PIR1 & CCP1IF) && (PIE1 & CCP1IE)) {   // embOS Timer
    _OS_SystemIrqhandler();     
  }
#if (OS_UART_USED)
  if ((PIR1 & TX1IF) && (PIE1 & TX1IE)) {     // Uart0 Tx
    OS_COM_ISR_Tx();  
  }
  if ((PIR1 & RC1IF) && (PIE1 & RC1IE)) {     // Uart0 Rx
    OS_COM_ISR_Rx();
  }
#endif
 OS_UserIrqfunc();                            // User defined
#if (ALLOW_NESTED_INTERRUPTS == 1)
  OS_LeaveNestableInterrupt();
#else
  OS_LeaveInterrupt();
#endif  
}

/*****  End Of File *************************************************/

